/// @file   Adapter.h
/// @brief  Dummy Plugin concrete factories.
/// @author Alejandro Álvarez Ayllón <aalvarez@cern.ch>
#ifndef ADAPTER_H
#define	ADAPTER_H

#include <dmlite/cpp/authn.h>
#include <dmlite/cpp/catalog.h>
#include <dmlite/cpp/dmlite.h>
#include <dmlite/cpp/poolmanager.h>

namespace dmlite {

  /// Concrete factory for DPNS/LFC wrapper
  class NsAdapterFactory: public CatalogFactory, public AuthnFactory {
  public:
    /// Constructor
    NsAdapterFactory() throw (DmException);
    /// Destructor
    ~NsAdapterFactory();

    void configure(const std::string& key, const std::string& value) throw (DmException);

    Catalog* createCatalog(PluginManager*) throw (DmException);  
    Authn*   createAuthn  (PluginManager*) throw (DmException);

  protected:
    unsigned retryLimit_;
    bool hostDnIsRoot_;
    std::string hostDn_;
  };

  /// Concrete factory for DPM wrapper
  class DpmAdapterFactory: public NsAdapterFactory, public PoolManagerFactory, public PoolDriverFactory {
  public:
    /// Constructor
    DpmAdapterFactory() throw (DmException);
    /// Destructor
    ~DpmAdapterFactory();

    void configure(const std::string& key, const std::string& value) throw (DmException);

    Catalog*     createCatalog(PluginManager*)     throw (DmException);
    PoolManager* createPoolManager(PluginManager*) throw (DmException);

    std::string implementedPool() throw();
    PoolDriver* createPoolDriver(void) throw (DmException);

  protected:
    unsigned retryLimit_;

    std::string tokenPasswd_;
    bool        tokenUseIp_;
    unsigned    tokenLife_;
  };

};

#endif	// ADAPTER_H

