/// @file   RFIO.h
/// @brief  RFIO IO.
/// @author CERN DPM <dpm-devel@cern.ch>
#ifndef RFIO_H
#define	RFIO_H

#include <dmlite/cpp/dmlite.h>
#include <dmlite/cpp/io.h>
#include <fstream>

namespace dmlite {

  class StdRFIOFactory: public IOFactory {
   public:
    StdRFIOFactory();
    ~StdRFIOFactory();

    void configure(const std::string& key, const std::string& value) throw (DmException);
    IODriver* createIODriver(PluginManager* pm) throw (DmException);

   private:
    std::string passwd_;
    bool        useIp_;
  };

  class StdRFIODriver: public IODriver {
   public:
    StdRFIODriver(std::string passwd, bool useIp);
    ~StdRFIODriver();

    std::string getImplId() const throw();

    void setStackInstance(StackInstance* si) throw (DmException);
    void setSecurityContext(const SecurityContext* ctx) throw (DmException);

    IOHandler* createIOHandler(const std::string& pfn, int flags,
                               const Extensible& extras, mode_t mode) throw (DmException);

    void doneWriting(const Location& loc) throw (DmException);

   private:
    StackInstance*         si_;
    const SecurityContext* secCtx_;

    std::string passwd_;
    bool        useIp_;
  };

  class StdRFIOHandler: public IOHandler {
  public:
    StdRFIOHandler(const std::string& path,
                   int flags, mode_t mode) throw (DmException);
    ~StdRFIOHandler();
    
    void   close(void) throw (DmException);
    size_t read (char* buffer, size_t count) throw (DmException);
    size_t write(const char* buffer, size_t count) throw (DmException);
    void   seek (off_t offset, Whence whence) throw (DmException);
    off_t  tell (void) throw (DmException);
    void   flush(void) throw (DmException);
    bool   eof  (void) throw (DmException);

  protected:
    int  fd_;
    bool eof_;
    long pos_;
  };

};

#endif	// RFIO_H
